@@ -59,16 +59,15 @@ const size_t MAX_PEER_LIST_UPDATE = 100;
 } // namespace
 
 DefaultPeerStorage::DefaultPeerStorage()
-  : maxPeerListSize_(MAX_PEER_LIST_SIZE),
-    seederStateChoke_(new BtSeederStateChoke()),
-    leecherStateChoke_(new BtLeecherStateChoke()),
-    lastTransferStatMapUpdated_(0)
-{}
+    : maxPeerListSize_(MAX_PEER_LIST_SIZE),
+      seederStateChoke_(make_unique<BtSeederStateChoke>()),
+      leecherStateChoke_(make_unique<BtLeecherStateChoke>()),
+      lastTransferStatMapUpdated_(Timer::zero())
+{
+}
 
 DefaultPeerStorage::~DefaultPeerStorage()
 {
-  delete seederStateChoke_;
-  delete leecherStateChoke_;
   assert(uniqPeers_.size() == unusedPeers_.size() + usedPeers_.size());
 }
 
@@ -79,8 +78,8 @@ size_t DefaultPeerStorage::countAllPeer() const
 
 bool DefaultPeerStorage::isPeerAlreadyAdded(const std::shared_ptr<Peer>& peer)
 {
-  return uniqPeers_.count(std::make_pair(peer->getIPAddress(),
-                                         peer->getOrigPort()));
+  return uniqPeers_.count(
+      std::make_pair(peer->getIPAddress(), peer->getOrigPort()));
 }
 
 void DefaultPeerStorage::addUniqPeer(const std::shared_ptr<Peer>& peer)
@@ -90,20 +89,20 @@ void DefaultPeerStorage::addUniqPeer(const std::shared_ptr<Peer>& peer)
 
 bool DefaultPeerStorage::addPeer(const std::shared_ptr<Peer>& peer)
 {
-  if(isPeerAlreadyAdded(peer)) {
+  if (isPeerAlreadyAdded(peer)) {
     A2_LOG_DEBUG(fmt("Adding %s:%u is rejected because it has been already"
                      " added.",
                      peer->getIPAddress().c_str(), peer->getPort()));
     return false;
   }
-  if(isBadPeer(peer->getIPAddress())) {
+  if (isBadPeer(peer->getIPAddress())) {
     A2_LOG_DEBUG(fmt("Adding %s:%u is rejected because it is marked bad.",
                      peer->getIPAddress().c_str(), peer->getPort()));
     return false;
   }
   const size_t peerListSize = unusedPeers_.size();
-  if(peerListSize >= maxPeerListSize_) {
-    deleteUnusedPeer(peerListSize-maxPeerListSize_+1);
+  if (peerListSize >= maxPeerListSize_) {
+    deleteUnusedPeer(peerListSize - maxPeerListSize_ + 1);
   }
   unusedPeers_.push_front(peer);
   addUniqPeer(peer);
@@ -112,33 +111,36 @@ bool DefaultPeerStorage::addPeer(const std::shared_ptr<Peer>& peer)
   return true;
 }
 
-void DefaultPeerStorage::addPeer(const std::vector<std::shared_ptr<Peer> >& peers)
+void DefaultPeerStorage::addPeer(
+    const std::vector<std::shared_ptr<Peer>>& peers)
 {
   size_t added = 0;
   size_t addMax = std::min(maxPeerListSize_, MAX_PEER_LIST_UPDATE);
-  for(auto itr = peers.begin(),
-        eoi = peers.end(); itr != eoi && added < addMax; ++itr) {
-    const std::shared_ptr<Peer>& peer = *itr;
-    if(isPeerAlreadyAdded(peer)) {
+  for (auto itr = std::begin(peers), eoi = std::end(peers);
+       itr != eoi && added < addMax; ++itr) {
+    auto& peer = *itr;
+    if (isPeerAlreadyAdded(peer)) {
       A2_LOG_DEBUG(fmt("Adding %s:%u is rejected because it has been already"
                        " added.",
                        peer->getIPAddress().c_str(), peer->getPort()));
       continue;
-    } else if(isBadPeer(peer->getIPAddress())) {
+    }
+    else if (isBadPeer(peer->getIPAddress())) {
       A2_LOG_DEBUG(fmt("Adding %s:%u is rejected because it is marked bad.",
                        peer->getIPAddress().c_str(), peer->getPort()));
       continue;
-    } else {
-      A2_LOG_DEBUG(fmt(MSG_ADDING_PEER,
-                       peer->getIPAddress().c_str(), peer->getPort()));
+    }
+    else {
+      A2_LOG_DEBUG(
+          fmt(MSG_ADDING_PEER, peer->getIPAddress().c_str(), peer->getPort()));
     }
     unusedPeers_.push_front(peer);
     addUniqPeer(peer);
     ++added;
   }
   const size_t peerListSize = unusedPeers_.size();
-  if(peerListSize > maxPeerListSize_) {
-    deleteUnusedPeer(peerListSize-maxPeerListSize_);
+  if (peerListSize > maxPeerListSize_) {
+    deleteUnusedPeer(peerListSize - maxPeerListSize_);
   }
   A2_LOG_DEBUG(fmt("Now unused peer list contains %lu peers",
                    static_cast<unsigned long>(unusedPeers_.size())));
@@ -146,49 +148,44 @@ void DefaultPeerStorage::addPeer(const std::vector<std::shared_ptr<Peer> >& peer
 
 void DefaultPeerStorage::addDroppedPeer(const std::shared_ptr<Peer>& peer)
 {
-  // Make sure that duplicated peers exist in droppedPeers_. If
+  // Make sure that no duplicated peer exists in droppedPeers_. If
   // exists, erase older one.
-  for(auto i = droppedPeers_.begin(),
-        eoi = droppedPeers_.end(); i != eoi; ++i) {
-    if((*i)->getIPAddress() == peer->getIPAddress() &&
-       (*i)->getPort() == peer->getPort()) {
+  for (auto i = std::begin(droppedPeers_), eoi = std::end(droppedPeers_);
+       i != eoi; ++i) {
+    if ((*i)->getIPAddress() == peer->getIPAddress() &&
+        (*i)->getPort() == peer->getPort()) {
       droppedPeers_.erase(i);
       break;
     }
   }
   droppedPeers_.push_front(peer);
-  if(droppedPeers_.size() > 50) {
+  if (droppedPeers_.size() > 50) {
     droppedPeers_.pop_back();
   }
 }
 
-const std::deque<std::shared_ptr<Peer> >& DefaultPeerStorage::getUnusedPeers()
+const std::deque<std::shared_ptr<Peer>>& DefaultPeerStorage::getUnusedPeers()
 {
   return unusedPeers_;
 }
 
-const PeerSet& DefaultPeerStorage::getUsedPeers()
-{
-  return usedPeers_;
-}
+const PeerSet& DefaultPeerStorage::getUsedPeers() { return usedPeers_; }
 
-const std::deque<std::shared_ptr<Peer> >& DefaultPeerStorage::getDroppedPeers()
+const std::deque<std::shared_ptr<Peer>>& DefaultPeerStorage::getDroppedPeers()
 {
   return droppedPeers_;
 }
 
-bool DefaultPeerStorage::isPeerAvailable() {
-  return !unusedPeers_.empty();
-}
+bool DefaultPeerStorage::isPeerAvailable() { return !unusedPeers_.empty(); }
 
 bool DefaultPeerStorage::isBadPeer(const std::string& ipaddr)
 {
   auto i = badPeers_.find(ipaddr);
-  if(i == badPeers_.end()) {
+  if (i == std::end(badPeers_)) {
     return false;
   }
 
-  if(global::wallclock().getTime() >= (*i).second) {
+  if ((*i).second <= global::wallclock()) {
     badPeers_.erase(i);
     return false;
   }
@@ -198,14 +195,14 @@ bool DefaultPeerStorage::isBadPeer(const std::string& ipaddr)
 
 void DefaultPeerStorage::addBadPeer(const std::string& ipaddr)
 {
-  if(lastBadPeerCleaned_.difference(global::wallclock()) >= 3600) {
-    for(auto i = badPeers_.begin(),
-          eoi = badPeers_.end(); i != eoi;) {
-      if(global::wallclock().getTime() >= (*i).second) {
+  if (lastBadPeerCleaned_.difference(global::wallclock()) >= 1_h) {
+    for (auto i = std::begin(badPeers_); i != std::end(badPeers_);) {
+      if ((*i).second <= global::wallclock()) {
         A2_LOG_DEBUG(fmt("Purge %s from bad peer", (*i).first.c_str()));
         badPeers_.erase(i++);
         // badPeers_.end() will not be invalidated.
-      } else {
+      }
+      else {
         ++i;
       }
     }
@@ -213,12 +210,16 @@ void DefaultPeerStorage::addBadPeer(const std::string& ipaddr)
   }
   A2_LOG_DEBUG(fmt("Added %s as bad peer", ipaddr.c_str()));
   // We use variable timeout to avoid many bad peers wake up at once.
-  badPeers_[ipaddr] = global::wallclock().getTime()+
-    std::max(SimpleRandomizer::getInstance()->getRandomNumber(601), 120L);
+  auto t = global::wallclock();
+  t.advance(std::chrono::seconds(
+      std::max(SimpleRandomizer::getInstance()->getRandomNumber(601), 120L)));
+
+  badPeers_[ipaddr] = std::move(t);
 }
 
-void DefaultPeerStorage::deleteUnusedPeer(size_t delSize) {
-  for(; delSize > 0 && !unusedPeers_.empty(); --delSize) {
+void DefaultPeerStorage::deleteUnusedPeer(size_t delSize)
+{
+  for (; delSize > 0 && !unusedPeers_.empty(); --delSize) {
     onErasingPeer(unusedPeers_.back());
     unusedPeers_.pop_back();
   }
@@ -226,35 +227,39 @@ void DefaultPeerStorage::deleteUnusedPeer(size_t delSize) {
 
 std::shared_ptr<Peer> DefaultPeerStorage::checkoutPeer(cuid_t cuid)
 {
-  if(!isPeerAvailable()) {
+  if (!isPeerAvailable()) {
     return nullptr;
   }
-  std::shared_ptr<Peer> peer = unusedPeers_.front();
+  auto peer = unusedPeers_.front();
   unusedPeers_.pop_front();
+  if (peer->usedBy() != 0) {
+    A2_LOG_WARN(fmt("CUID#%" PRId64 " is already set for peer %s:%u",
+                    peer->usedBy(), peer->getIPAddress().c_str(),
+                    peer->getOrigPort()));
+  }
   peer->usedBy(cuid);
   usedPeers_.insert(peer);
   A2_LOG_DEBUG(fmt("Checkout peer %s:%u to CUID#%" PRId64,
-                   peer->getIPAddress().c_str(), peer->getPort(),
+                   peer->getIPAddress().c_str(), peer->getOrigPort(),
                    peer->usedBy()));
   return peer;
 }
 
 void DefaultPeerStorage::onErasingPeer(const std::shared_ptr<Peer>& peer)
 {
-  uniqPeers_.erase(std::make_pair(peer->getIPAddress(),
-                                  peer->getOrigPort()));
+  uniqPeers_.erase(std::make_pair(peer->getIPAddress(), peer->getOrigPort()));
 }
 
 void DefaultPeerStorage::onReturningPeer(const std::shared_ptr<Peer>& peer)
 {
-  if(peer->isActive()) {
-    if(peer->isDisconnectedGracefully() && !peer->isIncomingPeer()) {
+  if (peer->isActive()) {
+    if (peer->isDisconnectedGracefully() && !peer->isIncomingPeer()) {
       peer->startDrop();
       addDroppedPeer(peer);
     }
     // Execute choking algorithm if unchoked and interested peer is
     // disconnected.
-    if(!peer->amChoking() && peer->peerInterested()) {
+    if (!peer->amChoking() && peer->peerInterested()) {
       executeChoke();
     }
   }
@@ -264,44 +269,49 @@ void DefaultPeerStorage::onReturningPeer(const std::shared_ptr<Peer>& peer)
 void DefaultPeerStorage::returnPeer(const std::shared_ptr<Peer>& peer)
 {
   A2_LOG_DEBUG(fmt("Peer %s:%u returned from CUID#%" PRId64,
-                   peer->getIPAddress().c_str(), peer->getPort(),
+                   peer->getIPAddress().c_str(), peer->getOrigPort(),
                    peer->usedBy()));
-  if(usedPeers_.erase(peer)) {
+  if (usedPeers_.erase(peer)) {
     onReturningPeer(peer);
     onErasingPeer(peer);
-  } else {
-    A2_LOG_DEBUG(fmt("Cannot find peer %s:%u in usedPeers_",
-                     peer->getIPAddress().c_str(), peer->getPort()));
+  }
+  else {
+    A2_LOG_WARN(fmt("Cannot find peer %s:%u in usedPeers_",
+                    peer->getIPAddress().c_str(), peer->getOrigPort()));
   }
 }
 
 bool DefaultPeerStorage::chokeRoundIntervalElapsed()
 {
-  const time_t CHOKE_ROUND_INTERVAL = 10;
-  if(pieceStorage_->downloadFinished()) {
-    return seederStateChoke_->getLastRound().
-      difference(global::wallclock()) >= CHOKE_ROUND_INTERVAL;
-  } else {
-    return leecherStateChoke_->getLastRound().
-      difference(global::wallclock()) >= CHOKE_ROUND_INTERVAL;
+  constexpr auto CHOKE_ROUND_INTERVAL = 10_s;
+  if (pieceStorage_->downloadFinished()) {
+    return seederStateChoke_->getLastRound().difference(global::wallclock()) >=
+           CHOKE_ROUND_INTERVAL;
+  }
+  else {
+    return leecherStateChoke_->getLastRound().difference(global::wallclock()) >=
+           CHOKE_ROUND_INTERVAL;
   }
 }
 
 void DefaultPeerStorage::executeChoke()
 {
-  if(pieceStorage_->downloadFinished()) {
+  if (pieceStorage_->downloadFinished()) {
     return seederStateChoke_->executeChoke(usedPeers_);
-  } else {
+  }
+  else {
     return leecherStateChoke_->executeChoke(usedPeers_);
   }
 }
 
-void DefaultPeerStorage::setPieceStorage(const std::shared_ptr<PieceStorage>& ps)
+void DefaultPeerStorage::setPieceStorage(
+    const std::shared_ptr<PieceStorage>& ps)
 {
   pieceStorage_ = ps;
 }
 
-void DefaultPeerStorage::setBtRuntime(const std::shared_ptr<BtRuntime>& btRuntime)
+void DefaultPeerStorage::setBtRuntime(
+    const std::shared_ptr<BtRuntime>& btRuntime)
 {
   btRuntime_ = btRuntime;
 }
